//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  literunner.cpp
//  This is a part of the literunner source code.
//
//  Copyright (C) 2000-2001 MrJukes.
//                     2003 ilmcuts
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include <windows.h>
#include <string>
#include <vector>
#include <litestep/lsapi/lsapi.h>
#include "exports.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Prototypes
//
void ReadConfig();
void ClearConfig();
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// class OnRun
//
class OnRun
{
public:
    OnRun() : m_bRunning(false) { }

    bool IsRunning() const { return m_bRunning; }
    void SetRunning(bool bRunning) { m_bRunning = bRunning; }

	HWND hwnd;
    std::string classname, caption, cmd1, cmd2;

private:
    bool m_bRunning;

};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// typedefs
//
typedef std::vector<OnRun*> OnRunVector;


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals/Constants
//
const char szVersion[] = "LiteRunner 2.1 (MrJukes/ilmcuts)";
const char szAppName[] = "LiteRunner";

OnRunVector g_vecOnRun;

HINSTANCE hInstance;
HWND hwndMain, hwndParent;

bool g_bActive = true;

int msgs[] =
{
    LM_GETREVID,
    LM_REFRESH,
    0
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// !Bang handlers
//
void BangOn(HWND caller, const char* args)
{
	if (!g_bActive)
    {
        g_bActive = true;
        SetTimer(hwndMain, 0, 500, NULL);
    }
}

void BangOff(HWND caller, const char* args)
{
	if (g_bActive)
    {
        g_bActive = false;
        KillTimer(hwndMain, 0);
    }
}

void BangToggle(HWND caller, const char* args)
{
	if (g_bActive)
    {
        BangOff(caller, args);
    }
    else
    {
        BangOn(caller, args);
    }
}

void BangIdentify(HWND caller, const char* args)
{
	POINT p;
	GetCursorPos(&p);
	
    HWND hwnd = WindowFromPoint(p);
	
    if (hwnd)
	{
        char classname[256] = "";
        char caption[256] = "";
        char temp[550] = "";

        GetClassName(hwnd, classname, 256);
		GetWindowText(hwnd, caption, 256);

		sprintf(temp, "Class = %s\nCaption = %s", classname, caption);
		MessageBox(NULL, temp, szAppName, MB_SYSTEMMODAL | MB_ICONINFORMATION);
	}
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ReadConfig
//
void ReadConfig()
{
    FILE* step = LCOpen(NULL);

    char temp[MAX_LINE_LENGTH] = { 0 };
    
    while (LCReadNextConfig(step, "*OnRun", temp, MAX_LINE_LENGTH)) 
	{ 
        char token2[MAX_LINE_LENGTH] = { 0 };
        char token3[MAX_LINE_LENGTH] = { 0 };
        char token4[MAX_LINE_LENGTH] = { 0 };
        char token5[MAX_LINE_LENGTH] = { 0 };
        
        // *OnRun [Class][Caption][!OnOpen][!OnClose]
        char* tokens[] = { NULL, token2, token3, token4, token5 };

        int count = CommandTokenize(temp, tokens, 5, NULL);

		if (!strlen(token2))
        {
            continue;
        }
		
        OnRun* pOnRun = new OnRun();

        if (!pOnRun)
        {
            continue;
        }

        pOnRun->classname.assign(token2);

		if (strlen(token3))
        {
            pOnRun->caption.assign(token3);
        }
		
		if (strlen(token4))
        {
            pOnRun->cmd1.assign(token4);
        }   
		
		if (strlen(token5))
        {
            pOnRun->cmd2.assign(token5);
        }

        g_vecOnRun.push_back(pOnRun);
	}

	LCClose(step);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ClearConfig
//
void ClearConfig()
{
    for (OnRunVector::iterator iter = g_vecOnRun.begin(); iter != g_vecOnRun.end(); ++iter)
    {
        delete *iter;
    }

    g_vecOnRun.clear();
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// initModuleEx
//
int initModuleEx(HWND parent, HINSTANCE dllInst, LPCSTR szPath)
{
	hwndParent = parent;
	hInstance = dllInst;

    WNDCLASS wc = { 0 };
    wc.lpfnWndProc = WndProc;       // our window procedure
	wc.hInstance = dllInst;         // hInstance of DLL
	wc.lpszClassName = szAppName;   // our window class name
	
	if (!RegisterClass(&wc)) 
	{
		MessageBox(parent, "Error: Could not register window class",
            szAppName, MB_OK | MB_ICONERROR);

		return 1;
	}
 
    AddBangCommand("!LiteRunnerOn", BangOn);
    AddBangCommand("!LiteRunnerOff", BangOff);
    AddBangCommand("!LiteRunnerToggle", BangToggle);
    AddBangCommand("!LiteRunnerIdentify", BangIdentify);

    ReadConfig();

	hwndMain = CreateWindowEx(WS_EX_TOOLWINDOW, szAppName, NULL, WS_POPUP,
        0, 0, 0, 0, NULL, NULL, dllInst, NULL);
	
    if (!hwndMain)
    {
        MessageBox(parent, "Error: Could create main window",
            szAppName, MB_OK | MB_ICONERROR);

        return 2;
    }
	
	SendMessage(hwndParent, LM_REGISTERMESSAGE, (WPARAM)hwndMain, (LPARAM)msgs);
    
    SetTimer(hwndMain, 0, 500, NULL);	
	
    return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// quitModule
//
int quitModule(HINSTANCE dll)
{
	RemoveBangCommand("!LiteRunnerOn");
	RemoveBangCommand("!LiteRunnerOff");
	RemoveBangCommand("!LiteRunnerToggle");
	RemoveBangCommand("!LiteRunnerIdentify");

	SendMessage(hwndParent, LM_UNREGISTERMESSAGE, (WPARAM)hwndMain, (LPARAM)msgs);
	KillTimer(hwndMain, 0);

	ClearConfig();

    DestroyWindow(hwndMain);
	UnregisterClass(szAppName, dll);

	return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// WndProc
//
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case LM_GETREVID:
		{
			if (wParam == 0 || wParam == 1)
            {
                LPSTR buf = (LPSTR)lParam;
                strcpy(buf, szVersion);
                
                return strlen(buf);
            }

            return 0;
		}
		break;

        case LM_REFRESH:
        {
            ClearConfig();
            ReadConfig();
        }
        break;

		case WM_TIMER:
		{
            for (OnRunVector::iterator iter = g_vecOnRun.begin(); iter != g_vecOnRun.end(); ++iter)
            {
				if ((*iter)->IsRunning()) // Application is currently active
				{
					if (!IsWindow((*iter)->hwnd))
					{
						(*iter)->SetRunning(false);
						LSExecute(NULL, (*iter)->cmd2.c_str(), SW_SHOWNORMAL);

						(*iter)->hwnd = NULL;
					}
				}
				else // Application is not running
				{
					if ((*iter)->caption.empty())
                    {
                        (*iter)->hwnd = FindWindow((*iter)->classname.c_str(), NULL);
                    }
					else
                    {
                        (*iter)->hwnd = FindWindow((*iter)->classname.c_str(), (*iter)->caption.c_str());
                    }

					if ((*iter)->hwnd)
					{
						// Application has been started since last check
						(*iter)->SetRunning(true);
						LSExecute(NULL, (*iter)->cmd1.c_str(), SW_SHOWNORMAL);
					}
				}
			}
		}
		return 0;
	}

	return DefWindowProc(hwnd,msg, wParam,lParam);
}
